require('skyscrapers')

local skyscrapers = getSkyscraperDefinitions()
local maxSkyscraperSizeX = getBiggestSkyscraperX()
local maxSkyscraperSizeY = getBiggestSkyscraperY()
local maxSkyscraperSizeXY = math.max(maxSkyscraperSizeX,maxSkyscraperSizeY)
local spacingX = getSpacingX()
local spacingY = getSpacingY()
local premadeSkyscrapers = getPremadeSkyscraperDefinitions()

data:extend({
    
    {
        type = "item-group",
        name = "skyscrapers",
        order = "b-m",
        inventory_order = "d-m",
        icon = "__Skyscrapers__/graphics/icons/skyscraper_big.png",
        icon_size = 96,
    },
    {
        type = "item-subgroup",
        name = "skyscrapers-buildings",
        group = "skyscrapers",
        order = "a-a"
    },
    {
        type = "item-subgroup",
        name = "skyscrapers-buildings-unique",
        group = "skyscrapers",
        order = "b-a"
    },
    {
        type = "item-subgroup",
        name = "skyscrapers-buildings-premade",
        group = "skyscrapers",
        order = "c-a"
    },
    {
        type = "item-subgroup",
        name = "skyscrapers-floors",
        group = "skyscrapers",
        order = "d-a"
    },
    {
        type = "item-subgroup",
        name = "skyscrapers-elevators",
        group = "skyscrapers",
        order = "e-a"
    },
    {
        type = "item-subgroup",
        name = "skyscrapers-cables",
        group = "skyscrapers",
        order = "f-a"
    },
    })

local customOutOfMap = table.deepcopy(data.raw["tile"]["grass-1"])
customOutOfMap.collision_mask = data.raw["tile"]["out-of-map"].collision_mask
customOutOfMap.variants = data.raw["tile"]["out-of-map"].variants
customOutOfMap.name = "skyscraper-boundary"
data:extend({customOutOfMap})

--todo consider omitting them if 5dim is present (or done generically if a belt of sufficient length exists)
local longUndergrounds1 = table.deepcopy(data.raw["underground-belt"]["underground-belt"])
longUndergrounds1.name = "long-underground-belt"
longUndergrounds1.max_distance = 50
data:extend({longUndergrounds1})
local longUndergroundsItem1 = table.deepcopy(data.raw["item"]["underground-belt"])
longUndergroundsItem1.name = "long-underground-belt"
longUndergroundsItem1.place_result = "long-underground-belt"
data:extend({longUndergroundsItem1})
local longUndergroundsRecipe1 = table.deepcopy(data.raw["recipe"]["underground-belt"])
longUndergroundsRecipe1.name = "long-underground-belt"
longUndergroundsRecipe1.ingredients =
{
  {"iron-plate", 10},
  {"transport-belt", 50}
}
longUndergroundsRecipe1.result = "long-underground-belt"
data:extend({longUndergroundsRecipe1})
table.insert(data.raw["technology"]["logistics"].effects, {
    type = "unlock-recipe",
    recipe = "long-underground-belt"
})

local longUndergrounds2 = table.deepcopy(data.raw["underground-belt"]["fast-underground-belt"])
longUndergrounds2.name = "long-fast-underground-belt"
longUndergrounds2.max_distance = 50
data:extend({longUndergrounds2})
local longUndergroundsItem2 = table.deepcopy(data.raw["item"]["fast-underground-belt"])
longUndergroundsItem2.name = "long-fast-underground-belt"
longUndergroundsItem2.place_result = "long-fast-underground-belt"
data:extend({longUndergroundsItem2})
local longUndergroundsRecipe2 = table.deepcopy(data.raw["recipe"]["fast-underground-belt"])
longUndergroundsRecipe2.name = "long-fast-underground-belt"
longUndergroundsRecipe2.ingredients =
{
  {"iron-plate", 10},
  {"fast-transport-belt", 50}
}
longUndergroundsRecipe2.result = "long-fast-underground-belt"
data:extend({longUndergroundsRecipe2})
table.insert(data.raw["technology"]["logistics-2"].effects, {
    type = "unlock-recipe",
    recipe = "long-fast-underground-belt"
})

local longUndergrounds3 = table.deepcopy(data.raw["underground-belt"]["express-underground-belt"])
longUndergrounds3.name = "long-express-underground-belt"
longUndergrounds3.max_distance = 50
data:extend({longUndergrounds3})
local longUndergroundsItem3 = table.deepcopy(data.raw["item"]["express-underground-belt"])
longUndergroundsItem3.name = "long-express-underground-belt"
longUndergroundsItem3.place_result = "long-express-underground-belt"
data:extend({longUndergroundsItem3})
local longUndergroundsRecipe3 = table.deepcopy(data.raw["recipe"]["express-underground-belt"])
longUndergroundsRecipe3.name = "long-express-underground-belt"
longUndergroundsRecipe3.ingredients =
{
  {"iron-plate", 10},
  {"express-transport-belt", 50}
}
longUndergroundsRecipe3.result = "long-express-underground-belt"
data:extend({longUndergroundsRecipe3})
table.insert(data.raw["technology"]["logistics-3"].effects, {
    type = "unlock-recipe",
    recipe = "long-express-underground-belt"
})

local substation = table.deepcopy(data.raw["electric-pole"]["substation"])
substation.name = "skyscrapers-substation"
substation.order = "nowhere"
substation.minable.result = nil
substation.maximum_wire_distance = 1
substation.supply_area_distance = math.ceil(1+maxSkyscraperSizeXY/2.0)
substation.connection_points = {{ shadow = { copper = {0, 999}, green = {0, 999}, red = {0, 999}}, wire ={ copper = {0, 999}, green = {0, 999}, red = {0, 999}}}}
substation.pictures.direction_count = 1
substation.pictures.filename = "__Skyscrapers__/graphics/transparent.png"
substation.pictures.width = 32
substation.pictures.height = 32
substation.pictures.shift = {0.0, 0.0}
substation.collision_box = {{0.0, 0.0}, {0.0, 0.0}}
substation.selection_box = {{0, 0}, {0, 0}}
substation.collision_mask = {}
table.insert(substation.flags,"not-on-map")
data:extend({substation})

local pole = table.deepcopy(data.raw["electric-pole"]["medium-electric-pole"])
pole.name = "skyscrapers-pole"
pole.order = "nowhere"
pole.minable.result = nil
pole.maximum_wire_distance = math.ceil(spacingY / 2.0)
pole.supply_area_distance = 1
pole.connection_points = {{ shadow = { copper = {0, 999}, green = {0, 999}, red = {0, 999}}, wire ={ copper = {0, 999}, green = {0, 999}, red = {0, 999}}}}
pole.pictures.direction_count = 1
pole.pictures.filename = "__Skyscrapers__/graphics/transparent.png"
pole.pictures.width = 32
pole.pictures.height = 32
pole.pictures.shift = {0.0, 0.0}
pole.collision_box = {{0.0, 0.0}, {0.0, 0.0}}
pole.selection_box = {{0, 0}, {0, 0}}
pole.collision_mask = {}
table.insert(pole.flags,"not-on-map")
data:extend({pole})

local elevator = table.deepcopy(data.raw["car"]["car"])
elevator.name = "skyscrapers-elevator"
elevator.order = "nowhere"
elevator.minable.result = nil
elevator.collision_box = {{0.0, 0.0}, {0.0, 0.0}}
elevator.selection_box = {{0, 0}, {0, 0}}
elevator.collision_mask = {}
elevator.animation = {layers={{
frame_count = 1,
direction_count = 1,
width = 32,
height = 32,
filename = "__Skyscrapers__/graphics/transparent.png",
}}}
elevator.turret_animation = elevator.animation
elevator.guns = {}
table.insert(elevator.flags,"not-on-map")
elevator.working_sound.sound.volume = 0.0
elevator.working_sound.activate_sound.volume = 0.0
elevator.working_sound.deactivate_sound.volume = 0.0
elevator.light = {{
        type = "oriented",
        minimum_darkness = 0.3,
        picture =
        {
          filename = "__Skyscrapers__/graphics/transparent.png",
          priority = "extra-high",
          flags = { "light" },
          scale = 1,
          width = 32,
          height = 32
        },
        shift = {0, 0},
        size = 1,
        intensity = 1.0,
        color = {r = 1, g = 1, b = 1}
      }}
data:extend({elevator})

data:extend({{
    type = "sound",
    name = "elevator_ding",
    filename = "__base__/sound/programmable-speaker/vibraphone-36.ogg",
    volume = 1.0
}})

local function getPictures(name, sizeX, sizeY, sizeZ)

    res = {layers = {}}
    
    table.insert(res.layers, {
            filename = "__Skyscrapers__/graphics/entity/" ..name.. "/base.png",
                    priority = "extra-high",
                    width = sizeX*32,
                    height = 64,
                    shift = {0.0, (-1+sizeY/2) - 0}
        })
    for i = 1,sizeZ-1 do
        local tint = { r=1.0, g=1.0, b=1.0, a=0.8 }
        if i*2 < sizeY then tint.a = 1.0 end
                    
        table.insert(res.layers, {
            filename = "__Skyscrapers__/graphics/entity/" ..name.. "/floor.png",
                    priority = "extra-high",
                    width = sizeX*32,
                    height = 64,
                    tint = tint,
                    shift = {0.0, (-1+sizeY/2) - 2*i}
        })
    end
    table.insert(res.layers, {
            filename = "__Skyscrapers__/graphics/entity/" ..name.. "/top.png",
                    priority = "extra-high",
                    width = sizeX*32,
                    height = 96,
                    tint = { r=1.0, g=1.0, b=1.0, a=0.8 },
                    shift = {0.0, (-1+sizeY/2) - 2*(sizeZ) -0.5}
        })
    return res
    
end

local function makeSingleImageTile(tilename, filename, hrFileName)

    local newMain = 
    {
        {
            picture = filename,
            count = 1,
            size = 1
        }
    }
    if hrFileName then
        newMain = 
        {
            {
                picture = filename,
                count = 1,
                size = 1,
                hr_version =
                {
                    picture = hrFileName,
                    count = 1,
                    scale = 0.5,
                    size = 1
                }
            }
        }
    end

    return {
    name = tilename,
    type = "tile",
    collision_mask =
    {
      "ground-tile",
      "item-layer",
      "object-layer",
    },
    layer = 70,
    variants =
    {
      main = newMain,
      inner_corner =
      {
        picture = "",
        count = 0
      },
      outer_corner =
      {
        picture = "",
        count = 0
      },
      side =
      {
        picture = "",
        count = 0
      }
    },
    map_color={r=0, g=0, b=1},
    ageing=0.0006
  }
end

data:extend({
    makeSingleImageTile("skyscraper-exit", "__base__/graphics/icons/blueprint.png"),
    makeSingleImageTile("skyscraper-stairs-up", "__Skyscrapers__/graphics/tiles/StairsUp.png"),
    makeSingleImageTile("skyscraper-stairs-down", "__Skyscrapers__/graphics/tiles/StairsDown.png"),
    makeSingleImageTile("skyscraper-elevator-entrance-1", "__Skyscrapers__/graphics/tiles/door1/door1.png", "__Skyscrapers__/graphics/tiles/door1/hr-door1.png"),
    makeSingleImageTile("skyscraper-elevator-entrance-2", "__Skyscrapers__/graphics/tiles/door1/door2.png", "__Skyscrapers__/graphics/tiles/door1/hr-door2.png"),
    makeSingleImageTile("skyscraper-elevator-entrance-3", "__Skyscrapers__/graphics/tiles/door1/door3.png", "__Skyscrapers__/graphics/tiles/door1/hr-door3.png"),
    makeSingleImageTile("skyscraper-elevator-entrance-4", "__Skyscrapers__/graphics/tiles/door1/door4.png", "__Skyscrapers__/graphics/tiles/door1/hr-door4.png"),

    {
        type = "item",
        name = "skyscraper-template",
        icon = "__Skyscrapers__/graphics/icons/skyscraper_small.png",
        icon_size = 32,
        flags = {"goes-to-main-inventory"},
        subgroup = "skyscrapers-buildings",
        order = "a",
        place_result = "skyscraper-template",
        stack_size = 10
    },
    
    {
        type = "recipe",
        name = "skyscraper-template",
        enabled = "true",
        category = "crafting",
        ingredients =
        {
            --{name = "raw-wood", amount = 20},
            --{name = "stone", amount = 20},
        },
        energy_required = 1,
        result = "skyscraper-template",
        result_count = 1
    },
    
    {
        type = "electric-energy-interface",
        name = "skyscraper-template",
        icon = "__Skyscrapers__/graphics/icons/skyscraper_small.png",
        icon_size = 32,
        order="e-a-b",
        flags = {"placeable-neutral", "player-creation", "not-blueprintable", "not-rotatable", "not-deconstructable"},
        minable = {hardness = 0.2, mining_time = 0.5, result = "skyscraper-template"},
        max_health = 150,
        corpse = "medium-remnants",
        collision_box = {{-4.9, -3.9}, {4.9, 3.9}},
        selection_box = {{-5.0, -4.0}, {5.0, 4.0}},
        energy_source =
        {
            type = "electric",
            buffer_capacity = "10MJ",
            usage_priority = "secondary-input",
            input_flow_limit = "10MW",
            output_flow_limit = "1kW"
        },
        pictures = getPictures("skyscraper-template", 10, 8, 5),
        charge_animation =
        {
            filename = "__Skyscrapers__/graphics/entity/mine-entrance/mine-entrance-charge-animation.png",
            width = 160,
            height = 112,
            line_length = 1,
            frame_count = 1,
            shift = {0.625, -0.125},
            animation_speed = 0.5
        },
        charge_cooldown = 30,
        charge_light = {intensity = 0.3, size = 7, color = {r = 1.0, g = 1.0, b = 1.0}},
        discharge_animation =
        {
            filename = "__Skyscrapers__/graphics/entity/mine-entrance/mine-entrance-discharge-animation.png",
            width = 160,
            height = 112,
            line_length = 1,
            frame_count = 1,
            shift = {0.625, -0.125},
            animation_speed = 0.5
        },
        discharge_cooldown = 60,
        discharge_light = {intensity = 0.7, size = 7, color = {r = 1.0, g = 1.0, b = 1.0}},
        vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
        working_sound =
        {
            sound =
            {
                filename = "__base__/sound/accumulator-working.ogg",
                volume = 1
            },
            idle_sound = {
                filename = "__base__/sound/accumulator-idle.ogg",
                volume = 0.4
            },
            max_sounds_per_type = 5
        },
        circuit_wire_connection_point =
        {
            shadow =
            {
                red = {0.984375, 1.10938},
                green = {0.890625, 1.10938}
            },
            wire =
            {
                red = {0.6875, 0.59375},
                green = {0.6875, 0.71875}
            }
        },
        circuit_connector_sprites = circuit_connector_definitions["belt"].sprites,
        circuit_wire_max_distance = 9,
        default_output_signal = {type = "virtual", name = "signal-A"},
    },
})

for name,templateData in pairs(skyscrapers) do

    local skyscraperIcons = nil
    if not templateData.unique and templateData.allowedCraftingCategories then 
        skyscraperIcons = {
            {
                icon = "__Skyscrapers__/graphics/icons/skyscraper_small.png",
                scale = 1.0,
                icon_size = 32,
            },
            {
                icon = data.raw.item[templateData.restrictionItemName].icon,
                scale = 0.5,
                shift = {4, -8},
                icon_size = data.raw.item[templateData.restrictionItemName].icon_size,
            }
        }
    end

    local skyscraperEntity = table.deepcopy(data.raw["electric-energy-interface"]["skyscraper-template"])
    skyscraperEntity.name = name
    skyscraperEntity.minable.result = name
    skyscraperEntity.pictures = getPictures(name, templateData.X, templateData.Y, templateData.Z)
    if skyscraperIcons then skyscraperEntity.icon = nil skyscraperEntity.icon_size = nil skyscraperEntity.icons = skyscraperIcons end
    skyscraperEntity.collision_box = {{0.1-templateData.X/2.0, 0.1-templateData.Y/2.0}, {-0.1+templateData.X/2.0,-0.1+templateData.Y/2.0}}
    skyscraperEntity.selection_box = {{-templateData.X/2.0, -templateData.Y/2.0}, {templateData.X/2.0, templateData.Y/2.0}}
    skyscraperEntity.energy_source.buffer_capacity = templateData.energyBuffer
    skyscraperEntity.energy_source.input_flow_limit = templateData.energyFlow
    skyscraperEntity.energy_source.output_flow_limit = templateData.energyFlow
    local restrictionStr = ""
    if templateData.allowedCraftingCategories then for _,v in pairs(templateData.allowedCraftingCategories) do restrictionStr = restrictionStr .. v .. "," end end
    if string.len(restrictionStr) > 0 then restrictionStr = string.sub(restrictionStr, 1, string.len(restrictionStr)-1) end
    if not templateData.unique and not templateData.allowedCraftingCategories then skyscraperEntity.localised_name = {"item-name.skyscraper-unrestricted",tostring(templateData.Z)} end
    if not templateData.unique and templateData.allowedCraftingCategories then skyscraperEntity.localised_name = {"item-name.skyscraper-restricted",tostring(templateData.Z), restrictionStr} end
    if templateData.unique and not templateData.allowedCraftingCategories then skyscraperEntity.localised_name = {"item-name.skyscraper-unique-unrestricted",templateData.uniqueName, tostring(templateData.Z)} end
    if templateData.unique and templateData.allowedCraftingCategories then skyscraperEntity.localised_name = {"item-name.skyscraper-unique-unrestricted",templateData.uniqueName, tostring(templateData.Z), restrictionStr} end
    if not templateData.unique and not templateData.allowedCraftingCategories then skyscraperEntity.localised_description = {"item-description.skyscraper-unrestricted",tostring(templateData.Z)} end
    if not templateData.unique and templateData.allowedCraftingCategories then skyscraperEntity.localised_description = {"item-description.skyscraper-restricted",tostring(templateData.Z), restrictionStr} end
    if templateData.unique and not templateData.allowedCraftingCategories then skyscraperEntity.localised_description = {"item-description.skyscraper-unique-unrestricted",templateData.uniqueDescription, tostring(templateData.Z)} end
    if templateData.unique and templateData.allowedCraftingCategories then skyscraperEntity.localised_description = {"item-description.skyscraper-unique-unrestricted",templateData.uniqueDescription, tostring(templateData.Z), restrictionStr} end
    if templateData.unique then skyscraperEntity.icon = "__Skyscrapers__/graphics/icons/"..name.."/icon.png" end
    if templateData.unique then skyscraperEntity.icon_size = 64 end
    data:extend({skyscraperEntity})

    local skyscraperRecipe = table.deepcopy(data.raw.recipe["skyscraper-template"])
    skyscraperRecipe.name = name
    skyscraperRecipe.result = name
    local numCombinators = templateData.Z
    if templateData.elevator then numCombinators = numCombinators + templateData.Z end
    skyscraperRecipe.ingredients = {
        {name = "skyscraper-floor-"..templateData.tileCraft.."-"..tostring(templateData.X).."-"..tostring(templateData.Y), amount = templateData.Z + 1},
        {name = "small-lamp", amount = templateData.Z},
        {name = "constant-combinator", amount = numCombinators},
        {name = "SNTD-nixie-tube", amount = templateData.Z * 2},
    }
    if templateData.elevator then table.insert(skyscraperRecipe.ingredients, {name = "SNTD-nixie-tube-small", amount = templateData.Z}) end
    if templateData.elevator then table.insert(skyscraperRecipe.ingredients, {name = "elevator-"..tostring(templateData.Z), amount = 1}) end
    
    data:extend({skyscraperRecipe})
    
    local skyscraperItem = table.deepcopy(data.raw.item["skyscraper-template"])
    skyscraperItem.name = name
    skyscraperItem.result = name
    skyscraperItem.place_result = name
    skyscraperItem.localised_name = skyscraperEntity.localised_name
    if templateData.unique then skyscraperItem.subgroup = "skyscrapers-buildings-unique" end
    if templateData.unique then skyscraperItem.icon = "__Skyscrapers__/graphics/icons/"..name.."/icon.png" end
    if templateData.unique then skyscraperItem.icon_size = 64 end
    if skyscraperIcons then skyscraperItem.icon = nil skyscraperItem.icon_size = nil skyscraperItem.icons = skyscraperIcons end
    data:extend({skyscraperItem})
            
    
    local skyscraperAccumulator = table.deepcopy(data.raw["accumulator"]["accumulator"])
    skyscraperAccumulator.name = name .. "-accumulator"
    skyscraperAccumulator.order = "nowhere"
    skyscraperAccumulator.minable.result = nil
    skyscraperAccumulator.energy_source.buffer_capacity = templateData.energyBuffer
    skyscraperAccumulator.energy_source.input_flow_limit = templateData.energyFlow
    skyscraperAccumulator.energy_source.output_flow_limit = templateData.energyFlow
    data:extend({skyscraperAccumulator})
    
    if not data.raw.item["skyscraper-floor-"..templateData.tileCraft.."-"..tostring(templateData.X).."-"..tostring(templateData.Y)] then
        data:extend({{
        type = "item",
        name = "skyscraper-floor-"..templateData.tileCraft.."-"..tostring(templateData.X).."-"..tostring(templateData.Y),
        icons = {
            {
                icon = "__Skyscrapers__/graphics/icons/floor.png",
                scale = 0.5,
                icon_size = 64,
            },
            {
                icon = data.raw.item[templateData.tileCraft].icon,
                scale = 0.5,
                shift = {4, -8},
                icon_size = data.raw.item[templateData.tileCraft].icon_size,
            }
        },
        flags = {"goes-to-main-inventory"},
        subgroup = "skyscrapers-floors",
        order = templateData.tileCraft .. string.format("%08d", templateData.X*templateData.Y),
        localised_name = {"item-name.skyscraper-floor",templateData.tileCraft,tostring(templateData.X),tostring(templateData.Y)},
        localised_description = {"item-name.skyscraper-floor",templateData.tileCraft,tostring(templateData.X),tostring(templateData.Y)},
        stack_size = 100
        }})
        data:extend({{
            type = "recipe",
            name = "skyscraper-floor-"..templateData.tileCraft.."-"..tostring(templateData.X).."-"..tostring(templateData.Y),
            enabled = "true",
            category = "crafting",
            ingredients =
            {
                {name = templateData.tileCraft, amount = templateData.X*templateData.Y},
            },
            energy_required = 1,
            result = "skyscraper-floor-"..templateData.tileCraft.."-"..tostring(templateData.X).."-"..tostring(templateData.Y),
            result_count = 1
        }})
    end
    if templateData.elevator and not data.raw.item["elevator-"..tostring(templateData.Z)] then
        data:extend({{
        type = "item",
        name = "elevator-"..tostring(templateData.Z),
        icon = "__Skyscrapers__/graphics/tiles/door1/hr-door2.png",
        icon_size = 64,
        flags = {"goes-to-main-inventory"},
        subgroup = "skyscrapers-elevators",
        order = string.format("%08d", templateData.Z),
        localised_name = {"item-name.elevator",tostring(templateData.Z)},
        localised_description = {"item-name.elevator",tostring(templateData.Z)},
        stack_size = 10
        }})
        local steelCableLength = templateData.Z*4*3 --2:1 elevator ratio would need roughly 4 times the cable length. also 3 meter per floor.
        local steelCableLengthStr = tostring(steelCableLength)
        data:extend({{
            type = "recipe",
            name = "elevator-"..tostring(templateData.Z),
            enabled = "true",
            category = "crafting",
            ingredients =
            {
                {name = "iron-plate", amount = 10},
                {name = "steel-plate", amount = 10},
                {name = "steel-cable-"..steelCableLengthStr, amount = 2},
                {name = "electronic-circuit", amount = templateData.Z + 5},
                {name = "electric-engine-unit", amount = 1},
            },
            energy_required = 1,
            result = "elevator-"..tostring(templateData.Z),
            result_count = 1
        }})
        if not data.raw.item["steel-cable-"..steelCableLengthStr] then
            data:extend({{
            type = "item",
            name = "steel-cable-"..steelCableLengthStr,
            icon = "__Skyscrapers__/graphics/icons/steel-wire.png",
            icon_size = 32,
            flags = {"goes-to-main-inventory"},
            subgroup = "skyscrapers-cables",
            order = string.format("%08d", steelCableLengthStr),
            localised_name = {"item-name.steel-cable",steelCableLengthStr},
            localised_description = {"item-name.steel-cable",steelCableLengthStr},
            stack_size = 10
            }})
            data:extend({{
                type = "recipe",
                name = "steel-cable-"..steelCableLengthStr,
                enabled = "true",
                category = "crafting",
                ingredients =
                {
                    {name = "steel-plate", amount = steelCableLength},
                },
                energy_required = 1,
                result = "steel-cable-"..steelCableLengthStr,
                result_count = 1
            }})
        end
    end

end
data.raw["electric-energy-interface"]["skyscraper-template"] = nil
data.raw.recipe["skyscraper-template"] = nil
data.raw.item["skyscraper-template"] = nil

for name,details in pairs(premadeSkyscrapers) do

    local baseTemplate = skyscrapers[details.base]
    local ingredientMap = {}
    for i = 1, baseTemplate.Z do
        local floorEntities = nil
        if i == 1 then 
            floorEntities = details.floor1 
        elseif i == baseTemplate.Z then
            floorEntities = details.floorTop
        else
            floorEntities = details.floorN
        end
        for _,v in pairs(floorEntities) do
            if not ingredientMap[v.name] then ingredientMap[v.name] = 0 end
            ingredientMap[v.name] = ingredientMap[v.name] + 1
        end
    end
    local ingredients = {}
    for k,v in pairs(ingredientMap) do
        table.insert(ingredients, {name = k, amount = v})
    end
    table.insert(ingredients, {name = details.base, amount = 1})

    data:extend({
      {
        type = "item",
        name = name,
        localised_name = {"item-name.skyscraper-prebuild",details.textName},
        localised_description = {"item-name.skyscraper-prebuild",details.textDescription},
        icons = {
            {
                icon = "__Skyscrapers__/graphics/icons/skyscraper_small.png",
                scale = 1.0,
                icon_size = 32,
            },
            {
                icon = data.raw.item[details.itemName].icon,
                scale = 0.5,
                shift = {4, -8},
                icon_size = data.raw.item[details.itemName].icon_size,
            }
        },
        flags = {"goes-to-quickbar"},
        subgroup = "skyscrapers-buildings-premade",
        order = "c[splitter]-a[splitter]",
        place_result = details.base,
        stack_size = 50
      },
          {
        type = "recipe",
        name = name,
        enabled = "true",
        category = "crafting",
        ingredients = ingredients,
        energy_required = 1,
        result = name,
        result_count = 1
    },
    })
    
end