--how far from each other each skyskraper is in the skyscraper surface, in x dimension
function getSpacingX()
    return 64
end

--how far each floor is apart from each other in the skyscraper surface, in y dimension
function getSpacingY()
    return 32
end
--X+Y: size of skyscraper in tiles in both surfaces
--Z: how many floors the skyscraper has
--entrancePos: entrance position relative to entity position 
--exitPos: position of the door in the skyscraper surface, relative to the lower left corner
--stairsUpPos: where the stairs going up are located
--stairsDownPos: where the stairs going down are located
--floorIndicatorPos: position of the entity that indicates the current floor. a constant combinator is created at y+1 and another digit display at x-1 too.
--elevatorPos: position where the tile is that allows entering the elevator. on y-1 a elevator indicater is placed, and at y-2 a combinator for it
--tile: what tiles to place on the ground    
function getSkyscraperDefinitions()
    local skyscrapers = {}
    skyscrapers["skyscraper-1-crafting"] =       { unique=false, X=10,Y=8, Z=5,  entrancePos={x=0.5,y=4}, exitPos={x=5.5,y=0}, 
        stairsUpPos={x=7,y=7},  stairsDownPos={x=2,y=7},  floorIndicatorPos={x=1,y=8},
        elevator=false, lampPos={x=5,y=10}, accumulatorPos={x=7,y=9}, energyBuffer="10MJ", energyFlow="2MW", tileCraft="stone-brick", tile="stone-path", 
        allowedCraftingCategories={"crafting", "advanced-crafting", "crafting-with-fluid"}, restrictionItemName="assembling-machine-1"}
    skyscrapers["skyscraper-1-smelting"] =       { unique=false, X=10,Y=8, Z=5,  entrancePos={x=0.5,y=4}, exitPos={x=5.5,y=0}, 
        stairsUpPos={x=7,y=7},  stairsDownPos={x=2,y=7},  floorIndicatorPos={x=1,y=8},
        elevator=false, lampPos={x=5,y=10}, accumulatorPos={x=7,y=9}, energyBuffer="10MJ", energyFlow="2MW", tileCraft="stone-brick",   tile="stone-path", 
        allowedCraftingCategories={"smelting"}, restrictionItemName="stone-furnace"}
    skyscrapers["skyscraper-1-oil-processing"] = { unique=false, X=10,Y=8, Z=5,  entrancePos={x=0.5,y=4}, exitPos={x=5.5,y=0}, 
        stairsUpPos={x=7,y=7},  stairsDownPos={x=2,y=7},  floorIndicatorPos={x=1,y=8},
        elevator=false, lampPos={x=5,y=10}, accumulatorPos={x=7,y=9}, energyBuffer="10MJ", energyFlow="2MW", tileCraft="stone-brick",   tile="stone-path", 
        allowedCraftingCategories={"oil-processing"}, restrictionItemName="oil-refinery"}
    skyscrapers["skyscraper-1-chemistry"] =      { unique=false, X=10,Y=8, Z=5,  entrancePos={x=0.5,y=4}, exitPos={x=5.5,y=0}, 
        stairsUpPos={x=7,y=7},  stairsDownPos={x=2,y=7},  floorIndicatorPos={x=1,y=8},
        elevator=false, lampPos={x=5,y=10}, accumulatorPos={x=7,y=9}, energyBuffer="10MJ", energyFlow="2MW", tileCraft="stone-brick",   tile="stone-path", 
        allowedCraftingCategories={"chemistry"}, restrictionItemName="chemical-plant"}
    skyscrapers["skyscraper-1-centrifuging"] =   { unique=false, X=10,Y=8, Z=5,  entrancePos={x=0.5,y=4}, exitPos={x=5.5,y=0}, 
        stairsUpPos={x=7,y=7},  stairsDownPos={x=2,y=7},  floorIndicatorPos={x=1,y=8},
        elevator=false, lampPos={x=5,y=10}, accumulatorPos={x=7,y=9}, energyBuffer="10MJ", energyFlow="2MW", tileCraft="stone-brick",   tile="stone-path", 
        allowedCraftingCategories={"centrifuging"}, restrictionItemName="centrifuge"}
    skyscrapers["skyscraper-2"] =                { unique=false, X=12,Y=10,Z=25, entrancePos={x=0.5,y=5}, exitPos={x=6.5,y=0}, 
        stairsUpPos={x=8,y=9},  stairsDownPos={x=3,y=9},  floorIndicatorPos={x=2,y=10}, 
        elevator=true, lampPos={x=6,y=12}, elevatorPos={x=6,y=9}, elevatorTopSpeed=0.6, accumulatorPos={x=8,y=11}, energyBuffer="50MJ", energyFlow="10MW", tileCraft="concrete",   tile="concrete"}
    skyscrapers["skyscraper-3"] =                { unique=false, X=14,Y=12,Z=50, entrancePos={x=0.5,y=6}, exitPos={x=7.5,y=0}, 
        stairsUpPos={x=9,y=11}, stairsDownPos={x=4,y=11}, floorIndicatorPos={x=3,y=12}, 
        elevator=true, lampPos={x=7,y=14}, elevatorPos={x=7,y=11}, elevatorTopSpeed=0.8, accumulatorPos={x=9,y=13}, energyBuffer="200MJ", energyFlow="40MW", tileCraft="refined-concrete",  tile="refined-concrete"}
    --mk4s, unique(only one instance at a time??, with cultural references
    --scale derived from https://vignette.wikia.nocookie.net/finalfantasy/images/1/11/Headquarters_f64.png/revision/latest?cb=20110517030706
    skyscrapers["skyscraper-shinra-tower"] =     { unique=true, uniqueName="Shinra Headquarters", uniqueDescription="The Headquarters of the Shinra Electric Power Company, also known as Shinra Inc.", X=38,Y=26,Z=70,  entrancePos={x=0.5,y=13}, exitPos={x=19.5,y=0}, 
        stairsUpPos={x=21,y=25}, stairsDownPos={x=16,y=25}, floorIndicatorPos={x=15,y=26}, 
        elevator=true, lampPos={x=19,y=28}, elevatorPos={x=19,y=25}, elevatorTopSpeed=1.0, accumulatorPos={x=21,y=29}, energyBuffer="1000MJ", energyFlow="250MW", tileCraft="refined-concrete",  tile="refined-concrete"}
    return skyscrapers
end

function getHighestSkyscraper()
    local maxZ = 0
    for _,val in pairs(getSkyscraperDefinitions()) do
        maxZ = math.max(maxZ,val.Z)
    end
    return maxZ
end

function getBiggestSkyscraperX()
    local maxX = 0
    for _,val in pairs(getSkyscraperDefinitions()) do
        maxX = math.max(maxX,val.X)
    end
    return maxX
end

function getBiggestSkyscraperY()
    local maxY = 0
    for _,val in pairs(getSkyscraperDefinitions()) do
        maxY = math.max(maxY,val.Y)
    end
    return maxY
end

function getPremadeSkyscraperDefinitions()
    skyscrapers = {}
    skyscrapers["skyscrapers-premade-gears-mk1"] = {
        base="skyscraper-1-crafting", 
        itemName="iron-gear-wheel",
        textName="simple gears, v1",
        textDescription="2 yellow belts input left for 19.33 iron plates/s, 1 yellow belt output right with 9.67 iron gears/s. No inserter stacksize bonus assumed.",
        floor1={
            {name="transport-belt",x=0,y=0,direction=defines.direction.north},
            {name="splitter",x=0,y=1,direction=defines.direction.north},
            {name="transport-belt",x=0,y=2,direction=defines.direction.north},
            {name="transport-belt",x=0,y=3,direction=defines.direction.north},
            {name="transport-belt",x=0,y=4,direction=defines.direction.north},
            {name="long-underground-belt",x=0,y=5,direction=defines.direction.north,underground_type="input"},
            {name="transport-belt",x=1,y=0,direction=defines.direction.north},
            {name="transport-belt",x=1,y=2,direction=defines.direction.north},
            {name="transport-belt",x=1,y=3,direction=defines.direction.north},
            {name="transport-belt",x=1,y=4,direction=defines.direction.north},
            {name="long-underground-belt",x=1,y=5,direction=defines.direction.north,underground_type="input"},
            {name="inserter",x=2,y=0,direction=defines.direction.west},
            {name="inserter",x=2,y=1,direction=defines.direction.west},
            {name="inserter",x=2,y=2,direction=defines.direction.west},
            {name="inserter",x=2,y=3,direction=defines.direction.west},
            {name="inserter",x=2,y=4,direction=defines.direction.west},
            {name="inserter",x=2,y=5,direction=defines.direction.west},
            {name="assembling-machine-1",x=4,y=1,recipe="iron-gear-wheel"},
            {name="assembling-machine-1",x=4,y=4,recipe="iron-gear-wheel"},
            {name="inserter",x=6,y=0,direction=defines.direction.west},
            {name="inserter",x=6,y=1,direction=defines.direction.west},
            {name="inserter",x=6,y=3,direction=defines.direction.west},
            {name="inserter",x=6,y=4,direction=defines.direction.west},
            {name="transport-belt",x=7,y=0,direction=defines.direction.south},
            {name="underground-belt",x=7,y=1,direction=defines.direction.south,underground_type="output"},
            {name="inserter",x=7,y=2,direction=defines.direction.north},
            {name="underground-belt",x=7,y=4,direction=defines.direction.south,underground_type="input"},
            {name="long-underground-belt",x=7,y=5,direction=defines.direction.south,underground_type="output"},
        },
        floorN={
            {name="long-underground-belt",x=0,y=0,direction=defines.direction.north,underground_type="output"},
            {name="splitter",x=0,y=1,direction=defines.direction.north},
            {name="transport-belt",x=0,y=2,direction=defines.direction.north},
            {name="transport-belt",x=0,y=3,direction=defines.direction.north},
            {name="transport-belt",x=0,y=4,direction=defines.direction.north},
            {name="long-underground-belt",x=0,y=5,direction=defines.direction.north,underground_type="input"},
            {name="long-underground-belt",x=1,y=0,direction=defines.direction.north,underground_type="output"},
            {name="transport-belt",x=1,y=2,direction=defines.direction.north},
            {name="transport-belt",x=1,y=3,direction=defines.direction.north},
            {name="transport-belt",x=1,y=4,direction=defines.direction.north},
            {name="long-underground-belt",x=1,y=5,direction=defines.direction.north,underground_type="input"},
            {name="inserter",x=2,y=0,direction=defines.direction.west},
            {name="inserter",x=2,y=1,direction=defines.direction.west},
            {name="inserter",x=2,y=2,direction=defines.direction.west},
            {name="inserter",x=2,y=3,direction=defines.direction.west},
            {name="inserter",x=2,y=4,direction=defines.direction.west},
            {name="inserter",x=2,y=5,direction=defines.direction.west},
            {name="assembling-machine-1",x=4,y=1,recipe="iron-gear-wheel"},
            {name="assembling-machine-1",x=4,y=4,recipe="iron-gear-wheel"},
            {name="inserter",x=6,y=0,direction=defines.direction.west},
            {name="inserter",x=6,y=1,direction=defines.direction.west},
            {name="inserter",x=6,y=3,direction=defines.direction.west},
            {name="inserter",x=6,y=4,direction=defines.direction.west},
            {name="long-underground-belt",x=7,y=0,direction=defines.direction.south,underground_type="input"},
            {name="underground-belt",x=7,y=1,direction=defines.direction.south,underground_type="output"},
            {name="inserter",x=7,y=2,direction=defines.direction.north},
            {name="underground-belt",x=7,y=4,direction=defines.direction.south,underground_type="input"},
            {name="long-underground-belt",x=7,y=5,direction=defines.direction.south,underground_type="output"},
        },
        floorTop={
            {name="long-underground-belt",x=0,y=0,direction=defines.direction.north,underground_type="output"},
            {name="splitter",x=0,y=1,direction=defines.direction.north},
            {name="long-underground-belt",x=1,y=0,direction=defines.direction.north,underground_type="output"},
            {name="transport-belt",x=1,y=2,direction=defines.direction.north},
            {name="transport-belt",x=1,y=3,direction=defines.direction.north},
            {name="transport-belt",x=1,y=4,direction=defines.direction.north},
            {name="transport-belt",x=1,y=5,direction=defines.direction.north},
            {name="inserter",x=2,y=0,direction=defines.direction.west},
            {name="inserter",x=2,y=1,direction=defines.direction.west},
            {name="inserter",x=2,y=2,direction=defines.direction.west},
            {name="inserter",x=2,y=3,direction=defines.direction.west},
            {name="inserter",x=2,y=4,direction=defines.direction.west},
            {name="inserter",x=2,y=5,direction=defines.direction.west},
            {name="assembling-machine-1",x=4,y=1,recipe="iron-gear-wheel"},
            {name="assembling-machine-1",x=4,y=4,recipe="iron-gear-wheel"},
            {name="inserter",x=6,y=0,direction=defines.direction.west},
            {name="transport-belt",x=6,y=1,direction=defines.direction.east},
            {name="transport-belt",x=6,y=2,direction=defines.direction.south},
            {name="transport-belt",x=6,y=3,direction=defines.direction.south},
            {name="inserter",x=6,y=4,direction=defines.direction.west},
            {name="inserter",x=6,y=5,direction=defines.direction.west},
            {name="long-underground-belt",x=7,y=0,direction=defines.direction.south,underground_type="input"},
            {name="transport-belt",x=7,y=1,direction=defines.direction.south},
            {name="transport-belt",x=7,y=2,direction=defines.direction.north},
            {name="transport-belt",x=7,y=3,direction=defines.direction.west},
            {name="transport-belt",x=7,y=4,direction=defines.direction.south},
            {name="transport-belt",x=7,y=5,direction=defines.direction.south},
        }
    }
    skyscrapers["skyscrapers-premade-gears-mk2"] = {
        base="skyscraper-1-crafting", 
        itemName="iron-gear-wheel",
        textName="simple gears, v2",
        textDescription="2 yellow belts input left for 26 iron plates/s, 1 yellow belt output right with 13 iron gears/s. Assumes inserter stacksize bonus 1 to reach its target output (otherwise 8.3 /s).",
        floor1={
            {name="transport-belt",x=0,y=0,direction=defines.direction.north},
            {name="splitter",x=0,y=1,direction=defines.direction.north},
            {name="transport-belt",x=0,y=2,direction=defines.direction.north},
            {name="transport-belt",x=0,y=3,direction=defines.direction.north},
            {name="transport-belt",x=0,y=4,direction=defines.direction.north},
            {name="long-underground-belt",x=0,y=5,direction=defines.direction.north,underground_type="input"},
            {name="transport-belt",x=1,y=0,direction=defines.direction.north},
            {name="transport-belt",x=1,y=2,direction=defines.direction.north},
            {name="transport-belt",x=1,y=3,direction=defines.direction.north},
            {name="transport-belt",x=1,y=4,direction=defines.direction.north},
            {name="long-underground-belt",x=1,y=5,direction=defines.direction.north,underground_type="input"},
            {name="inserter",x=2,y=1,direction=defines.direction.west},
            {name="inserter",x=2,y=2,direction=defines.direction.west},
            {name="inserter",x=2,y=3,direction=defines.direction.west},
            {name="inserter",x=2,y=4,direction=defines.direction.west},
            {name="assembling-machine-2",x=4,y=1,recipe="iron-gear-wheel"},
            {name="assembling-machine-2",x=4,y=4,recipe="iron-gear-wheel"},
            {name="inserter",x=6,y=1,direction=defines.direction.west},
            {name="inserter",x=6,y=3,direction=defines.direction.west},
            {name="transport-belt",x=7,y=0,direction=defines.direction.south},
            {name="underground-belt",x=7,y=1,direction=defines.direction.south,underground_type="output"},
            {name="inserter",x=7,y=2,direction=defines.direction.north},
            {name="iron-chest",x=7,y=3,inventory_bar=1},
            {name="underground-belt",x=7,y=4,direction=defines.direction.south,underground_type="input"},
            {name="long-underground-belt",x=7,y=5,direction=defines.direction.south,underground_type="output"},
        },
        floorN={
            {name="long-underground-belt",x=0,y=0,direction=defines.direction.north,underground_type="output"},
            {name="splitter",x=0,y=1,direction=defines.direction.north},
            {name="transport-belt",x=0,y=2,direction=defines.direction.north},
            {name="transport-belt",x=0,y=3,direction=defines.direction.north},
            {name="transport-belt",x=0,y=4,direction=defines.direction.north},
            {name="long-underground-belt",x=0,y=5,direction=defines.direction.north,underground_type="input"},
            {name="long-underground-belt",x=1,y=0,direction=defines.direction.north,underground_type="output"},
            {name="transport-belt",x=1,y=2,direction=defines.direction.north},
            {name="transport-belt",x=1,y=3,direction=defines.direction.north},
            {name="transport-belt",x=1,y=4,direction=defines.direction.north},
            {name="long-underground-belt",x=1,y=5,direction=defines.direction.north,underground_type="input"},
            {name="inserter",x=2,y=1,direction=defines.direction.west},
            {name="inserter",x=2,y=2,direction=defines.direction.west},
            {name="inserter",x=2,y=3,direction=defines.direction.west},
            {name="inserter",x=2,y=4,direction=defines.direction.west},
            {name="assembling-machine-2",x=4,y=1,recipe="iron-gear-wheel"},
            {name="assembling-machine-2",x=4,y=4,recipe="iron-gear-wheel"},
            {name="inserter",x=6,y=1,direction=defines.direction.west},
            {name="inserter",x=6,y=3,direction=defines.direction.west},
            {name="long-underground-belt",x=7,y=0,direction=defines.direction.south,underground_type="input"},
            {name="underground-belt",x=7,y=1,direction=defines.direction.south,underground_type="output"},
            {name="inserter",x=7,y=2,direction=defines.direction.north},
            {name="iron-chest",x=7,y=3,inventory_bar=1},
            {name="underground-belt",x=7,y=4,direction=defines.direction.south,underground_type="input"},
            {name="long-underground-belt",x=7,y=5,direction=defines.direction.south,underground_type="output"},
        },
        floorTop={
            {name="long-underground-belt",x=0,y=0,direction=defines.direction.north,underground_type="output"},
            {name="splitter",x=0,y=1,direction=defines.direction.north},
            {name="long-underground-belt",x=1,y=0,direction=defines.direction.north,underground_type="output"},
            {name="transport-belt",x=1,y=2,direction=defines.direction.north},
            {name="transport-belt",x=1,y=3,direction=defines.direction.north},
            {name="transport-belt",x=1,y=4,direction=defines.direction.north},
            {name="inserter",x=2,y=1,direction=defines.direction.west},
            {name="inserter",x=2,y=2,direction=defines.direction.west},
            {name="inserter",x=2,y=3,direction=defines.direction.west},
            {name="inserter",x=2,y=4,direction=defines.direction.west},
            {name="assembling-machine-2",x=4,y=1,recipe="iron-gear-wheel"},
            {name="assembling-machine-2",x=4,y=4,recipe="iron-gear-wheel"},
            {name="inserter",x=6,y=1,direction=defines.direction.west},
            {name="inserter",x=6,y=3,direction=defines.direction.west},
            {name="long-underground-belt",x=7,y=0,direction=defines.direction.south,underground_type="input"},
            {name="transport-belt",x=7,y=1,direction=defines.direction.south},
            {name="inserter",x=7,y=2,direction=defines.direction.north},
            {name="iron-chest",x=7,y=3,inventory_bar=1},
        }
    }
    return skyscrapers
end
